/**
 *
 * File information:<br>
 * $Author: nmk148 $<br>
 * First check-in: $Date: 2003/04/01 05:11:49 $<br>
 * $Revision: 1.4 $<br>
 * <br>
 *
 * This class encapsulates the main algorithm to solve 
 * problems using GA, ASGA or Hybrid.
 * <ol>
 *   <li>Create a population</li>
 *   <li>Run through many generation cycles</li>
 *   <li>Stop at a certain condition</li>
 * </ol>
 * The class can be inherited in order to change the 
 * stop condition.
 * The standard condition is to perform a certain number
 * of generation cycles. The number is given to the constructor.
 **/
public class Solver {



	/* Fields */
	private int generations;
	private Population pop;
	
	
	/**
	 * Create a new solver with the given parameters
	 * @param generations number of generations to run
	 * @param popSize population size
	 * @param problem the problem specification
	 **/
	public Solver(int generations, Population pop) {
		this.pop = pop;
		this.generations = generations;
	}

	/**
	 * Solves the problem. Runs through the generation cycles.
	 **/
	public void solve() {
		for (int i=0; i<generations; i++) {
			pop.nextGeneration();
			generationDone();
		}
	}

	/**
	 * This method is called when a generation is over.
	 * It can be used to print out the stastical data.
	 **/
	public void generationDone() {
		pop.computeStats();
		System.out.println(pop.getBest().fitness()+
		              "\t"+pop.getAverageFitness()+
			      "\t"+pop.getStdDevFitness());
	}

}
/*
 * History of this file:
 * $Log: Solver.java,v $
 * Revision 1.4  2003/04/01 05:11:49  nmk148
 * Removed unnecessary printout statements
 *
 * Revision 1.3  2003/03/28 16:12:22  nmk148
 * Finished
 *
 * Revision 1.2  2003/03/26 18:46:33  nmk148
 * Program done
 *
 * Revision 1.1  2003/03/23 17:21:26  nmk148
 * First checkin
 *
 */
